﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Emgu.CV;
using Emgu.CV.Structure;
using Emgu.CV.CvEnum;
using FeatureDetectors;

namespace features {
	public partial class ImageChangeDialog : Form {

		public Emgu.CV.Image<Gray, byte> _srcImg;
		public Emgu.CV.Image<Gray, byte> _destImg;
		private Emgu.CV.Image<Gray, byte> _noiseImg;
		public bool _createNew = false;
		public ImageProperties imageProps;
		public ImageChangeDialog() {	
			InitializeComponent();
			imageProps = new ImageProperties();
		}



		private void clearBrightness_Click(object sender, EventArgs e) {
			trackBrightness.Value = 0;
			changeImage();
		}
		private void clearContrast_Click(object sender, EventArgs e) {
			trackContrast.Value = 0;
			changeImage();
		}
		private void clearBlur_Click(object sender, EventArgs e) {
			trackBlur.Value = 0;
			changeImage();
		}
		private void clearRotation_Click(object sender, EventArgs e) {
			trackRotation.Value = 0;
			changeImage();
		}
		private void clearScale_Click(object sender, EventArgs e) {
			trackScale.Value = 0;
			changeImage();
		}
		private void clearNoise_Click(object sender, EventArgs e) {
			trackNoise.Value = 0;
			changeImage();
		}

		private void trackBrightness_ValueChanged(object sender, EventArgs e) {
			inputBrightness.Text = trackBrightness.Value.ToString();
			clearBrightness.Visible = trackBrightness.Value != 0;

			imageProps.brightness = trackBrightness.Value;
			changeImage();
		}
		private void trackNoise_ValueChanged(object sender, EventArgs e) {
			inputNoise.Text = trackNoise.Value.ToString();
			clearNoise.Visible = trackNoise.Value != 0;
			imageProps.noise = trackNoise.Value;
			changeImage();
		}

		private void trackScale_ValueChanged(object sender, EventArgs e) {
			inputScale.Text = trackScale.Value.ToString();
			clearScale.Visible = trackScale.Value != 0;
			imageProps.scale = trackScale.Value;

			changeImage();
		}

		private void trackRotation_ValueChanged(object sender, EventArgs e) {
			inputRotation.Text = trackRotation.Value.ToString();
			clearRotation.Visible = trackRotation.Value != 0;
			imageProps.rotation = trackRotation.Value;
			changeImage();
		}

		private void trackBlur_ValueChanged(object sender, EventArgs e) {
			inputBlur.Text = trackBlur.Value.ToString();
			clearBlur.Visible = trackBlur.Value != 0;
			imageProps.blur = trackBlur.Value;

			changeImage();
		}

		private void trackContrast_ValueChanged(object sender, EventArgs e) {
			inputContrast.Text = trackContrast.Value.ToString();
			clearContrast.Visible = trackContrast.Value != 0;
			imageProps.contrast = trackContrast.Value;
			changeImage();
		}

		private void button1_MouseDown(object sender, MouseEventArgs e) {
			pictureBox1.Image = _srcImg.ToBitmap();
		}

		private void changeImage() {
			if ( _destImg == null || _srcImg == null ) return;
			if ( _noiseImg == null ) {
				_noiseImg = _srcImg.Clone();
				_noiseImg.SetRandUniform(new MCvScalar(0), new MCvScalar(255));
			}
			try {
				Image<Gray, byte> noiseImg = _noiseImg.Clone();

				double contrast = (double) trackContrast.Value;
				contrast =  contrast /100 + 1;

				_destImg = _srcImg.Clone();
				if (trackScale.Value != 0) {
					double scale = trackScale.Value > 0 ? trackScale.Value / 100.0 * 3 + 1 : 1 + trackScale.Value / 110.0;
					_destImg = _destImg.Resize((int) Math.Floor(_srcImg.Width * scale), (int) Math.Floor(_srcImg.Height * scale), INTER.CV_INTER_LINEAR);
					noiseImg = noiseImg.Resize((int) Math.Floor(_srcImg.Width * scale), (int) Math.Floor(_srcImg.Height * scale), INTER.CV_INTER_NN);
				}
				if (trackBlur.Value>0) _destImg = _destImg.SmoothGaussian(2*(trackBlur.Value / 2) +1);
				_destImg += noiseImg * (trackNoise.Value / 100.0);
				_destImg -= noiseImg.Not() * (trackNoise.Value / 100.0);
				_destImg *= contrast;
				_destImg += (1 - contrast) / 2 + trackBrightness.Value;

				_destImg = _destImg.Rotate((double) trackRotation.Value, new Gray(0), false);



				pictureBox1.Image = _destImg.ToBitmap();
			}
			catch {
				MessageBox.Show("Error occured", "Error", MessageBoxButtons.OK);
			}
		}

		private void button1_MouseUp(object sender, MouseEventArgs e) {
			pictureBox1.Image = _destImg.ToBitmap();
		}
		public  DialogResult ShowModal(Image<Gray, Byte> image, string imageID) {
			imageProps.previousImageID = imageID;
			_srcImg = image.Clone();
			_destImg = image.Clone();
			_noiseImg = null;
			changeImage();
			image = _destImg;
			return this.ShowDialog();
		}

		private void okButton_Click(object sender, EventArgs e) {
			_srcImg = _destImg;
		}
		public void resetTracks() {			
			trackNoise.Value = 0;
			trackContrast.Value = 0;
			trackBrightness.Value = 0;
			trackBlur.Value = 0;
			trackRotation.Value = 0;
			trackScale.Value = 0;
		}

		private void checkBox1_CheckedChanged(object sender, EventArgs e) {
			_createNew = checkBox1.Checked;
		}

		private void button2_Click(object sender, EventArgs e) {
			resetTracks();
		}

		private void inputBrightness_TextChanged(object sender, EventArgs e) {
			try { 
				int value = Convert.ToInt32( inputBrightness.Text);
				value = Math.Min(Math.Max(value, -100), 100);
				inputBrightness.Text = value.ToString();
				trackBrightness.Value = value;
			}
			catch { }
		}

		private void inputContrast_TextChanged(object sender, EventArgs e) {
			try {
				int value = Convert.ToInt32(inputContrast.Text);
				value = Math.Min(Math.Max(value, -100), 100);
				inputContrast.Text = value.ToString();
				trackContrast.Value = value;
			}
			catch { }
		}

		private void inputBlur_TextChanged(object sender, EventArgs e) {
			try {
				int value = Convert.ToInt32(inputBlur.Text);
				value = Math.Min(Math.Max(value, 0), 100);
				inputBlur.Text = value.ToString();
				trackBlur.Value = value;
			}
			catch { }
		}

		private void inputRotation_TextChanged(object sender, EventArgs e) {
			try {
				int value = Convert.ToInt32(inputRotation.Text);
				value = Math.Min(Math.Max(value, -180), 180);
				inputRotation.Text = value.ToString();
				trackRotation.Value = value;
			}
			catch { }
		}

		private void inputScale_TextChanged(object sender, EventArgs e) {
			try {
				int value = Convert.ToInt32(inputScale.Text);
				value = Math.Min(Math.Max(value, -100), 100);
				inputScale.Text = value.ToString();
				trackScale.Value = value;
			}
			catch { }
		}

		private void inputNoise_TextChanged(object sender, EventArgs e) {
			try {
				int value = Convert.ToInt32(inputNoise.Text);
				value = Math.Min(Math.Max(value, 0), 100);
				inputNoise.Text = value.ToString();
				trackNoise.Value = value;
			}
			catch { }
		}


	}
	
		
}
