﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
using System.Diagnostics;
using System.Windows.Forms;
using Emgu.CV;
using Emgu.CV.Structure;
using FeatureDetectors;

namespace features
{
    public partial class MainForm : Form
    {

		private ImageChangeDialog imgChangeDg;
		private List<CDetector> featureMethods;
        public MainForm()
        {
        	InitializeComponent();
			imgChangeDg = new ImageChangeDialog();
			featureMethods = new List<CDetector>();
			string[] files = Directory.GetFiles(Application.StartupPath + @"\features_dll\", "*.dll");

			foreach ( string f in files ) {
				LoadDll(Application.StartupPath + @"\features_dll\" + Path.GetFileName(f));
			}
        }
		private void LoadDll(string fileName) {
			try {
				Assembly u = Assembly.LoadFile(fileName);
				Type[] tp = u.GetTypes();
				Type type = u.GetType("FeatureDetectors.CDetectorFunc");
				if (type != null) {
					CDetector featureDetector = (CDetector) Activator.CreateInstance(type);
					
					featureMethods.Add(featureDetector);
					int index = featureMethods.IndexOf(featureDetector);

					ToolStripMenuItem featureMenu = (ToolStripMenuItem) menuMethods.DropDownItems.Add(featureDetector.detectorName);
					featureMenu.Checked = true;
					featureMenu.Tag = index;
					featureMenu.CheckOnClick = true;
					ToolStripMenuItem featureOptMenu1 = (ToolStripMenuItem) featureMenu.DropDownItems.Add("Run");
					featureOptMenu1.Click += new EventHandler(OnFeatureRunClick);
					ToolStripMenuItem featureOptMenu2 = (ToolStripMenuItem) featureMenu.DropDownItems.Add("Run for checked");
					featureOptMenu2.Click += new EventHandler(OnFeatureRunAllClick);
					ToolStripMenuItem featureOptMenu3 = (ToolStripMenuItem) featureMenu.DropDownItems.Add("Options");
					featureOptMenu3.Click += new EventHandler(OnFeatureOptionsClick);
				}
			}
			catch {
				MessageBox.Show("Failed loading methods", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		public ToolStripMenuItem AddMdiMenu(MDIChildForm child) {
			ToolStripMenuItem wndMenu = (ToolStripMenuItem) menuWindows.DropDownItems.Add(child.Text);
			wndMenu.Tag = child;
			wndMenu.Click += new EventHandler(OnWndClick);
			return wndMenu;
		}

		protected void OnWndClick(object sender, EventArgs e) {
			ToolStripMenuItem optionsMenu = (ToolStripMenuItem) sender;
			MDIChildForm child = (MDIChildForm) optionsMenu.Tag;
			if (child != null) {
				child.Activate();
			}
		}
		private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
			Application.Exit();
		}

		private void loadImageToolStripMenuItem_Click(object sender, EventArgs e) {
			opDgOpen.ShowDialog();
		}

		private void opDgOpen_FileOk(object sender, CancelEventArgs e) {
			if ( opDgOpen.FileName != "" ) {
				MDIChildForm newMDIForm = new MDIChildForm(this, opDgOpen.FileName);
			}
		}

		private void cacadeToolStripMenuItem_Click(object sender, EventArgs e) {
			this.LayoutMdi(MdiLayout.Cascade);
		}

		private void horizontalToolStripMenuItem_Click(object sender, EventArgs e) {
			this.LayoutMdi(MdiLayout.TileHorizontal);
		}

		private void verticalToolStripMenuItem_Click(object sender, EventArgs e) {
			this.LayoutMdi(MdiLayout.TileVertical);
		}



		private void changeImageToolStripMenuItem_Click(object sender, EventArgs e) {
			MDIChildForm imgForm = (MDIChildForm) this.ActiveMdiChild;
			if (imgForm != null)
				switch ( imgChangeDg.ShowModal(imgForm.image, imgForm.imageProperties.imageID)) {
					case DialogResult.OK:
						if ( imgChangeDg._createNew ) {
							MDIChildForm ff = new MDIChildForm(this, imgChangeDg._destImg, imgChangeDg.imageProps);
						}
						else imgForm.loadImage(imgChangeDg._destImg);
						break;

				} 
		}

		private void MainForm_MdiChildActivate(object sender, EventArgs e) {
			int c = 6;
		}
		protected void OnMethodsSelect(object sender, EventArgs e) {
			

		}
		protected void OnFeatureOptionsClick(object sender, EventArgs e) {
			ToolStripMenuItem optionsMenu = (ToolStripMenuItem) sender;
			int index = (int) optionsMenu.OwnerItem.Tag;

			if ( featureMethods[index] != null ) {
				featureMethods[index].ShowOptionsDialog();

			}
		}
		protected void OnFeatureRunClick(object sender, EventArgs e) {
			ToolStripMenuItem optionsMenu = (ToolStripMenuItem) sender;
			int index = (int) optionsMenu.OwnerItem.Tag;
			if ( featureMethods[index] != null ) {
				MDIChildForm mdiChild = (MDIChildForm) this.ActiveMdiChild;
				if ( mdiChild != null ) FindDrawFeatures(mdiChild, featureMethods[index]);
			}
		}
		private void FindDrawFeatures(MDIChildForm mdiChild, CDetector detector) {
			Image<Bgr, byte> image;
			DrawType drawPointType  = DrawType.DOT;
			image = mdiChild.image.Convert<Bgr, byte>();

			mdiChild.results = new Results(detector, mdiChild.imageProperties);
			mdiChild.results.Start();
			mdiChild.results.features = detector.GetFeatures(mdiChild.image);
			mdiChild.results.Stop();
			detector.GenerateDestriptors(mdiChild.results.features, mdiChild.image);
			detector.DrawFeatures(
				image,
				mdiChild.results.features,
				drawPointType 
			);



			mdiChild.displayImage(image);

		}



		protected void OnFeatureRunAllClick(object sender, EventArgs e) {
			ToolStripMenuItem optionsMenu = (ToolStripMenuItem) sender;
			int index = (int) optionsMenu.OwnerItem.Tag;
			
			if ( featureMethods[index] != null ) {
				foreach ( MDIChildForm mdiChild in this.MdiChildren ) {
					if ( mdiChild.includeInList ) {
						FindDrawFeatures(mdiChild, featureMethods[index]);
					}
				}
			}
		}

		private void cloneImageToolStripMenuItem_Click(object sender, EventArgs e) {
			MDIChildForm imgForm = (MDIChildForm) this.ActiveMdiChild;
			if (imgForm != null) new MDIChildForm(this, imgForm.image);
		}

		private void menuMethods_Click(object sender, EventArgs e) {

		}

		private void toolStripMenuItem2_Click(object sender, EventArgs e) {
			foreach (MDIChildForm mdiChild in this.MdiChildren) {
				if (mdiChild.results != null) {
					mdiChild.results.Save();
					mdiChild.SaveImages();
				}
			}
		}
    }
}
